#if !defined(GLOBAL_H_)
#define GLOBAL_H_

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdint.h>

#define nop() asm volatile ("nop")


//
// PIN DEFINITIONS
//

#define HSYNC_DDR   DDRB
#define HSYNC_PORT  PORTB
#define HSYNC_PIN   PB1 // OC1A

#define VSYNC_DDR   DDRB
#define VSYNC_PORT  PORTB
#define VSYNC_PIN   PB3 // OC2A

#define COLOR_DDR	DDRC
#define COLOR_PORT	PORTC

#define SHIFT_PORT PORTB
#define SHIFT_DDR  DDRB
#define SHIFT_PL   PB2
#define SHIFT_CE   PB0

//
// VGA TIMING
//

// CVT
// mode:        640x400 @ 75Hz
// pixel clock: 25MHz
//
// http://www.vesa.org/Public/CVT/CVTd6r1.xls
//
// unit: characters = PIXELS*8
#define H_FRONT   2  // front porch
#define H_SYNC    8  // sync pulse -- NEGATIVE polarity
#define H_BACK    10  // back porch
#define H_ACTIVE  80 // active video
#define H_TOTAL   (H_SYNC+H_BACK+H_ACTIVE+H_FRONT)
#define H_CHARS   H_ACTIVE
// unit: LINES
#define V_FRONT   3   // front porch
#define V_SYNC    6   // sync pulse -- POSITIVE polarity
#define V_BACK    12  // back porch
#define V_ACTIVE  400 // active video
#define V_TOTAL   (V_SYNC+V_BACK+V_ACTIVE+V_FRONT)
#define V_CHARS   (((V_ACTIVE/8)/2) - 1)


//
// GLOBAL DATA
//


// FONT.C
extern const uint8_t font[256*8] PROGMEM;

// FRAMEBUFFER.C
extern volatile uint8_t framebuf[H_CHARS*V_CHARS];

// Keyboard konnections
#define KB_PORT  PORTD
#define KB_IN	 PIND
#define KB_DDR   DDRD
#define KB_DATA  PD2
#define KB_CLOCK PD3

#define KB_FRAMETIMER 32

#endif // GLOBAL_H_
